#
# Script to create schematic figure in Paper IV. 
#
# Sarah Brands, 2022
#
import os
import sys
import math
import numpy as np
import pandas as pd

from matplotlib import pyplot as plt
from scipy.interpolate import interp1d
from scipy.optimize import curve_fit
from scipy import odr

import sys
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.patches import Circle

import matplotlib

cmap = matplotlib.cm.get_cmap('inferno')
# cmap = matplotlib.cm.get_cmap('Spectral')


def E(q, r0, x, y):
    """Return the electric field vector E=(Ex,Ey) due to charge q at r0."""
    den = np.hypot(x-r0[0], y-r0[1])**3
    return q * (x - r0[0]) / den, q * (y - r0[1]) / den

# Numerical values are not representative of the models, 
# they are only for visualisation purposes! 

# Mean magnitude of magnetic field 
B0 = 3.12e-5
# Radius 
RE = 35.370
# Deviation of magnetic pole from axis
alpha = np.radians(0.0)

def B(r, theta):
    """Return the magnetic field vector at (r, theta)."""
    fac = B0 * (RE / r)**3
    return -2 * fac * np.cos(theta + alpha), -fac * np.sin(theta + alpha)

# Grid of x, y points on a Cartesian grid
nx, ny = 640, 640
XMAX, YMAX = 40, 40
x = np.linspace(-XMAX, XMAX, nx)
y = np.linspace(-YMAX, YMAX, ny)
X, Y = np.meshgrid(x, y)
r, theta = np.hypot(X, Y), np.arctan2(Y, X)

# Magnetic field vector, B = (Ex, Ey), as separate components
Br, Btheta = B(r, theta)
# Transform to Cartesian coordinates: NB make North point up, not to the right.
c, s = np.cos(np.pi/2 + theta), np.sin(np.pi/2 + theta)
Bx = -Btheta * s + Br * c
By = Btheta * c + Br * s

fig, ax = plt.subplots(1,2)

# Plot the streamlines with an appropriate colormap and arrow style
color = 'black'#2 * np.log(np.hypot(Bx, By))
ax[0].streamplot(x, y, Bx, By, color=color, linewidth=1, #cmap=plt.cm.inferno,
              density=1, arrowstyle='->', arrowsize=1.0)
ax[0].axis('off')


nsteps = 1000
radius = np.linspace(0,1,nsteps)
omega = np.zeros_like(radius)
# omega[omega<=0.8] = radius[omega<=0.8]*0.8
# omega[radius>0.8] = 1.0
# omega = omega + radius**0.5
omega = omega + 0.1
# omega[radius>0.8] = 0.5
# normalise omega
# omega_norm = (omega-np.min(omega))/np.max(omega)

for irad, iRE in zip(radius, omega):
    ax[0].add_patch(Circle((0,0), irad*RE, edgecolor=cmap(iRE), lw=1,  facecolor="none", zorder=0))

ax[0].set_xlabel('$x$')
ax[0].set_ylabel('$y$')
ax[0].set_xlim(-XMAX, XMAX)
ax[0].set_ylim(-YMAX, YMAX)
ax[0].set_aspect('equal')
ax[0].set_title('INT scheme')

cbar_ax = fig.add_axes([0.08, 0.1, 0.35, 0.03])
scat1 = ax[1].scatter([1e10,1e10,1e10],[1e10,1e10,1e10], c=np.linspace(-5.0,-4.3,3), cmap=cmap)
cbar1 = plt.colorbar(scat1, cax = cbar_ax, orientation='horizontal', ticks=[-6.0, -5.5, -5.0, -4.5])
cbar1.ax.set_title(r'log $\Omega$ [$\rm s^{-1}$]', fontsize=9)
# cbar1.ax.set_xticks([-6, -5, -4])
# cbar1.ax.set_xticklabels([-6, -5, -4])

# cbar1.ax.set_xticklabels(np.interp(cbar1.ax.get_xticks(), cbar1.ax.get_xlim(), cbar1.get_clim()).astype(int), fontsize=8)


nq = 2**int(2)
charges = []
for i in range(nq):
    q = i%2 * 2 - 1
    charges.append((q, (np.cos(2*np.pi*i/nq), np.sin(2*np.pi*i/nq))))

# Electric field vector, E=(Ex, Ey), as separate components
Ex, Ey = np.zeros((ny, nx)), np.zeros((ny, nx))
for charge in charges:
    ex, ey = E(*charge, x=X, y=Y)
    Ex += ex
    Ey += ey


ax[1].axis('off')
ax[1].set_xlim(-XMAX, XMAX)
ax[1].set_ylim(-YMAX, YMAX)
ax[1].set_aspect('equal')

ax[1].streamplot(x, y, Ex, Ey, color=color, linewidth=1, #cmap=plt.cm.inferno,
              density=1.2, arrowsize=0.8)


nsteps = 1000
radius = np.linspace(0,1,nsteps)
omega = np.zeros_like(radius)
omega = omega - radius**2
omega[radius>0.8] = radius[int(nsteps*0.8)]**2


# normalise omega
omega = (omega-np.min(omega))/np.max(omega)
# print omega
# Add a filled circle for the Earth; make sure it's on top of the streamlines.
# norm = 1.0*RE/256
# the_linspace = np.linspace(0, norm, 1000)/norm
# print omega
for irad, iRE in zip(radius, omega):
    ax[1].add_patch(Circle((0,0), irad*RE, edgecolor=cmap(iRE), lw=1,  facecolor="none", zorder=0))

ax[1].set_xlabel(r'$M(R)/\mathrm{M}_*$')
ax[1].set_title('SURF scheme')

cbar_ax = fig.add_axes([0.58, 0.1, 0.35, 0.03])

scat1 = ax[1].scatter([1e10,1e10,1e10,1e10],[1e10,1e10,1e10,1e10], c=np.linspace(-5.0,-4.3,4), cmap=cmap)

cbar1 = plt.colorbar(scat1, cax = cbar_ax, orientation='horizontal', ticks=[-6.0, -5.5, -5.0, -4.5])
cbar1.ax.set_title(r'log $\Omega$ [$\rm s^{-1}$]', fontsize=9)

plt.tight_layout()

plt.subplots_adjust(top=1.05, bottom=0.0, left=0.04, right=0.96)

plt.savefig('surfint.png')
plt.show()
