'''
This python script is used to read in MESA evolutionary files 
and create HRD, Kiel, and Hunter diagrams, displayed in 
"Paper IV" by Keszthelyi et al. 
Other diagnostics can also easily be set up with this architecture. 
The routine can be used for "production runs" up to plotting
all magnetic field strengths within 2 mixing schemes, 
which is 3 x 2 x 26 figures in one go. 
Nonetheless, the axes and labels may need to be adjusted
on a figure-by-figure basis.
In the current version, the metallicity and magnetic braking scheme
needs to be set in lines 52,53 and 57, respectively. For the NOMAG scheme,
the array in line XX needs to contain 609 for the magnetic field strength.
The path in line 61 needs to be set according to where the data is. 

Have fun plotting, 
    Zsolt Keszthelyi, 2022
'''

import matplotlib.pyplot as plt
import matplotlib.ticker as ticker
from matplotlib.ticker import FormatStrFormatter
import numpy as np
import sys
import math
from pylab import *
from matplotlib.collections import PatchCollection
from matplotlib.patches import Rectangle
from matplotlib.collections import LineCollection
from matplotlib.ticker import AutoMinorLocator

#----------------------------------------------
# some constanst and useful things
sigma = 5.67051 * pow(10,-5)  # cgs
Rsun = 6.957 * pow(10,10)     # cgs
Lsun = 3.828 * pow(10,33)     # cgs
pi = np.pi
G = 6.67259e-8 # cgs
Msun = 1.99e33  # cgs
year = 3.154e7  # sec. 
Teffsun = 5.780e3 # K 
loggsun = 4.43775 # cgs
Lscriptsun = pow(Teffsun,4.)/(10**loggsun)
#----------------------------------------------

def allplots(mix,Bpinin):
    
    np.seterr(divide='ignore', invalid='ignore')  

    #==== MODELS
    # === which metallicity? === 
    Zini = 0.014  # 0.014 or 0.0064 or 0.0026
    Zin = '14'    # 14 or 6 or 2 
    # === which B field strength? (set outside this def) === 
    Bpini = str(int(Bpinin*1000))
    # === which braking scheme? === 
    scheme = 'NOMAG'  # INT, SURF, NOMAG
    # === which initial masses? === 
    mod = ['3.0','4.0','5.0','6.0','7.0','8.0','9.0','10.0','11.0','12.0','13.0','14.0','15.0','16.0','17.0', '18.0', '19.0', '20.0', '21.0', '22.0','23.0','24.0','25.0', '30.0', '40.0', '50.0', '60.0']
    # === path to the data === 
    path = '/Volumes/Macintosh HD/Volumes/Extz/zsolt/RESEARCH/MESA/Grid/data/'+scheme+'/Z'+Zin+mix+'/'+Bpini+'/'
    # === only 'out.data' for RDM output files === 
    mes = '/LOGS/out.data'

    num = len(mod)

    k = ['navy','grey','green','yellow','blue','navy','grey','green','yellow','blue','navy','grey','green','yellow','blue','navy','grey','green','yellow','blue','navy','grey','green','yellow','blue']
    j = [':','--','-.','-',':','--','-.','-','--','-.','-','--','-.']
    lww = [6,6,6,12,12,12,18,18,18]

    # +++++++++++++++++++++ SETUP PLOTS HERE +++++++++++++++++++++
    rc('axes', linewidth=10)
    #rc('ticks', which = 'minor', width = 3, length = 12)
    # =======================================================================================
    # HRD === change axes per metallicity
    fig1, ax1 = plt.subplots(1,1, figsize=(20,20))
    ax1.set_xlim(56,5)     # for Solar and LMC/SMC Mix1 and INT Mix2
    #ax1.set_xlim(75,5)    # for LMC SURF Mix2 
    #:::ax1.set_xlim(60,5) # for SMC Mix1 / Mix2 60 - 80
    ax1.set_ylim(1.6,6.3)
    #ax1.set_ylim(4.15,4.99)
    ax1.set_xlabel('T$_{\mathrm{eff}}$ [kK]', size=60 )
    ax1.set_ylabel('log (L / L$_{\odot}$)', size=60 )
    ax1.tick_params(labelsize=54, width = 12, length = 22)
    ax1.xaxis.set_major_locator(ticker.MultipleLocator(5)) # 10)) ! 10 for some LMC/SMC Mix2
    ax1.xaxis.set_minor_locator(AutoMinorLocator(5))
    ax1.yaxis.set_minor_locator(AutoMinorLocator(5))
    ax1.tick_params(which = 'minor', width = 3, length = 12)
    # =======================================================================================
    # Kiel d. === change axes per metallicity
    fig111, ax111 = plt.subplots(1,1, figsize=(20,20))
    ax111.set_xlim(56,5)
    #:::ax111.set_xlim(60,5)
    ax111.invert_yaxis()
    #ax111.set_ylim(4.7,2.7)
    ax111.set_ylim(4.5,2.5)
    # tricky ::: ax111.set_ylim(4.5,0.9) # GALAXY: 0.9 for SURF/Mix2, 2.5 or so otherwise
    ax111.set_xlabel('T$_{\mathrm{eff}}$ [kK]', size=60 )
    ax111.set_ylabel('log g [cm s$^{-2}$]', size=60 )
    ax111.tick_params(labelsize=54, width = 12, length = 22)
    ax111.xaxis.set_major_locator(ticker.MultipleLocator(5)) # 5)) ! 10 for some LMC/SMC Mix2
    ax111.xaxis.set_minor_locator(AutoMinorLocator(5))
    ax111.yaxis.set_minor_locator(AutoMinorLocator(5))
    ax111.tick_params(which = 'minor', width = 3, length = 12)
    # =======================================================================================
    # Hunter === change axes per metallicity
    fig3, ax3 = plt.subplots(1,1, figsize=(20,20))
    ax3.set_xlim(-40,450)
    if Zin == '14' : 
    #   ax3.set_ylim(7.7,9.5) 
       ax3.set_ylim(7.7,9.5) 
    if Zin == '6' :     
       ax3.set_ylim(7.1,9.7)   # for Mix1 8.7 --- for Mix2 9.7)  
    if Zin == '2' :     
       ax3.set_ylim(6.7,9.4)   # Mix1: 8.7 Mix2: 9.4)         
    ax3.set_xlabel('$v_{\mathrm{rot}}$  [km s$^{-1}$]',  size=60)
    ax3.set_ylabel('log $[N/H] + 12$',  size=60)
    ax3.tick_params(labelsize=44, width = 9, length = 22)
    ax3.xaxis.set_minor_locator(AutoMinorLocator(5))
    ax3.yaxis.set_minor_locator(AutoMinorLocator(4))
    ax3.tick_params(which = 'minor', width = 3, length = 12)
    # =======================================================================================

    # This is for the colour bar if not set with fixed values. 
    # The overall min and max values will be found from a subgrid of all initial masses.
    # (For plots in Paper IV, we set fixed values for for the min and max on the colour bars, 
    # so that a direct comparison is feasible.)
    # Check the m/n_temp gtr / le signs depending on the parameter considered.

    # vrot
    def get_max_bar_value():
        m = 0
        for i in range(0,num):
            data = np.genfromtxt(path+mod[i]+mes, skip_header = 6)
            coreh = data[:,59]    # center H1
            xd = coreh.compress((coreh>coreh[0]*0.997).flat) 
            x = len(xd)
            vrot = data[x:,22]
            bar = vrot
            m_temp = bar.max()
            if m_temp > m:
                m = m_temp          
        return m
    m = get_max_bar_value()
    baz = m 

    def get_min_bar_value():
        n = 1e33  #0
        for i in range(0,num):
            data = np.genfromtxt(path+mod[i]+mes, skip_header = 6)
            coreh = data[:,59]    # center H1
            xd = coreh.compress((coreh>coreh[0]*0.997).flat) 
            x = len(xd)
            vrot = data[x:,22]
            bar = vrot
            n_temp = bar.min()
            if n_temp < n:  
                n = n_temp          
        return n
    n = get_min_bar_value()
    meg = n 
    
    # logg
    def get_max_bar_values():
        mm = 0
        for i in range(0,num):
            data = np.genfromtxt(path+mod[i]+mes, skip_header = 6)
            coreh = data[:,59]    # center H1
            xd = coreh.compress((coreh>coreh[0]*0.997).flat) 
            x = len(xd)
            logg =  data[x:,14]
            bar = logg
            m_temp = bar.max()
            if m_temp > mm:
                mm = m_temp          
        return mm
    mm = get_max_bar_values()
    bazz = mm 

    def get_min_bar_values():
        nn = 1e33
        for i in range(0,num):
            data = np.genfromtxt(path+mod[i]+mes, skip_header = 6)
            coreh = data[:,59]    # center H1
            xd = coreh.compress((coreh>coreh[0]*0.997).flat) 
            x = len(xd)
            logg =  data[x:,14]
            bar = logg
            n_temp = bar.min()
            if n_temp < nn:   
                nn = n_temp          
        return nn
    nn = get_min_bar_values()
    megg = nn 
   
    print ('************************************************************')
    print ('*   Thank you for plotting with python. Have a nice day.   *')
    print (' %s  models with %s kG field strength are now plotted.' %(mix,Bpinin))
    print ('************************************************************') 

    # empty vectors for iso-s
    logLzams = []
    loggzams = []
    Teffzams = []
    logLiso1 = []
    loggiso1 = []
    Teffiso1 = []
    logLiso2 = []
    loggiso2 = []
    Teffiso2 = []
    loggisox1 = []
    loggisox15 = []
    loggisox2 = []
    loggisox3 = []
    loggisox4 = []
    loggisox5 = []
    loggisox10 = []
    Teffisox1 = []
    Teffisox15 = []
    Teffisox2 = []
    Teffisox3 = []
    Teffisox4 = []
    Teffisox5 = []
    Teffisox10 = []
    # ---
    loggiso3 = []
    Teffiso3 = []
    loggiso5 = []
    Teffiso5 = []
    loggiso10 = []
    Teffiso10 = []
    loggiso20 = []
    Teffiso20 = []
    # ---
    logLiso4 = []
    Teffiso4 = []
    logLiso6 = []
    Teffiso6 = []
    logLiso8 = []
    Teffiso8 = []
    logLiso10 = []
    Teffiso10 = []
    logLiso50 = []
    Teffiso50 = []

    #===================================================================
    ##### read in the MESA files ###### 

    for i in range(0,num) : 
     data = np.genfromtxt(path+mod[i]+mes,skip_header=6)  
     #-----------------------------------------------
     coreh = data[:,59]
     xd = coreh.compress((coreh>coreh[0]*0.997).flat) # this is our ZAMS definition
     x = len(xd)
     model = data[:,0]
     m = len(model)
     age2 = data[x-1:m-1,1]
     age = data[x:,1]
     M = data[x:,2]
     logM = np.log10(M)
     Mdot = data[x:,3]
     Mdot = - Mdot 
     logMdot = np.log10(Mdot)
     logTeff = data[x:,6]
     Teff = data[x:,7]
     logL = data[x:,11]
     R =  data[x:,13]
     logg =  data[x:,14]
     g = 10**logg
     centernit = data[x:,62] # N14
     logsurfh1 = data[x:,69]
     surfh1 = pow(10,logsurfh1)
     nit = data[x:,66] # N14
     ox = data[x:,67] # O16
     car = data[x:,65]  # C12
     lognit = np.log10(nit/(14*surfh1)) + 12 # spectroscopic definition
     logsurfhe = data[x:,70] # He4 surf
     surfhe = 10**logsurfhe
     nith = nit/surfh1
     oxh = ox/surfh1
     carh = car/surfh1

     #-- rot
     vrot = data[x:,22]
     vcrit = data[x:,23]
     Jtot = data[x:,5]
     Omega = data[x:,19]
     Omegac = data[x:,58]
     omdivomcr = data[x:,20]
     periods = 2. * pi * (R*Rsun) / (vrot *1e5)
     period = periods / (3600. * 24.)
     logperiod = np.log10(period)

    #-- mag 
     Bp = data[x:,85]
     logBp = np.log10(Bp)
     etastar = data[x:,88]
     Mdotorig = data[x:,96]
     logMdotorig = np.log10(Mdotorig)
     Jbrake = data[x:,95]
     Jsurf = data[x:,99]
     logJsurf = np.log10(Jsurf)
     RA = data[x:,89]    # this is R_A / R_star
     Rc = data[x:,90]    # this is R_c / R_star
     gmr = G * (M*Msun) / (R*Rsun)   # cgs
     om2 = vrot*1.e5 / (R*Rsun) 
     Wpar = vrot*1.e5 / sqrt(gmr)    # cgs !!! 
     yy = - (2./3.)
     RK = pow(Wpar,yy)    # this is R_K / R_star
     RA = np.log10(RA)
     RK = np.log10(RK)
    #=========  END of MESA file input ==============================================================
     
    # set up more iso related things 

     #dd = Teff.compress(((age/1e6)<5).flat)
     #da = len(dd)
     if max(Omegac/Omega) > 1.0: 
      ddx1 = Teff.compress(((Omegac/Omega)<1.0).flat) 
      dax1 = len(ddx1)
      loggisox1.append(logg[dax1])
      Teffisox1.append(Teff[dax1]/1e3)

     if max(Omegac/Omega) > 1.2: 
      ddx15 = Teff.compress(((Omegac/Omega)<1.2).flat) 
      dax15 = len(ddx15)
      loggisox15.append(logg[dax15])
      Teffisox15.append(Teff[dax15]/1e3)

     if max(Omegac/Omega) > 2.: 
      ddx2 = Teff.compress(((Omegac/Omega)<2.).flat) 
      dax2 = len(ddx2)
      loggisox2.append(logg[dax2])
      Teffisox2.append(Teff[dax2]/1e3)

     if max(Omegac/Omega) > 3.: 
      ddx3 = Teff.compress(((Omegac/Omega)<3.).flat) 
      dax3 = len(ddx3)
      loggisox3.append(logg[dax3])
      Teffisox3.append(Teff[dax3]/1e3)

     if max(Omegac/Omega) > 4.: 
      ddx4 = Teff.compress(((Omegac/Omega)<4.).flat) 
      dax4 = len(ddx4)
      loggisox4.append(logg[dax4])
      Teffisox4.append(Teff[dax4]/1e3)

     if max(Omegac/Omega) > 5.: 
      ddx5 = Teff.compress(((Omegac/Omega)<5.).flat) 
      dax5 = len(ddx5)
      loggisox5.append(logg[dax5])
      Teffisox5.append(Teff[dax5]/1e3)

     if max(Omegac/Omega) > 10.: 
      ddx10 = Teff.compress(((Omegac/Omega)<10.).flat) 
      dax10 = len(ddx10)
      loggisox10.append(logg[dax10])
      Teffisox10.append(Teff[dax10]/1e3)


     dd1 = Teff.compress(((age/1e6)<2.).flat) # show 2 Myr
     da1 = len(dd1) 
     dd2 = Teff.compress(((age/1e6)<4.).flat) # show 4 Myr 
     da2 = len(dd2)
     dd3 = Teff.compress(((age/1e6)<6.).flat) # show 6 Myr
     da3 = len(dd3)
     dd4 = Teff.compress(((age/1e6)<8.).flat) # show 8 Myr
     da4 = len(dd4)
     dd5 = Teff.compress(((age/1e6)<10.).flat) # show 10 Myr
     da5 = len(dd5)
     dd50 = Teff.compress(((age/1e6)<50.).flat) # show 50 Myr
     da50 = len(dd50)

     logLzams.append(logL[0])
     Teffzams.append(Teff[0]/1e3)
     logLiso2.append(logL[da1])
     Teffiso2.append(Teff[da1]/1e3)
     
     if max(age/1e6) > 4: 
      logLiso4.append(logL[da2])
      Teffiso4.append(Teff[da2]/1e3)
     if max(age/1e6) > 6 :
      logLiso6.append(logL[da3])
      Teffiso6.append(Teff[da3]/1e3)
     if max(age/1e6) > 8 :
      logLiso8.append(logL[da4])
      Teffiso8.append(Teff[da4]/1e3)
     if max(age/1e6) > 10 :
      logLiso10.append(logL[da5])
      Teffiso10.append(Teff[da5]/1e3)
     if max(age/1e6) > 50 :
      logLiso50.append(logL[da50])
      Teffiso50.append(Teff[da50]/1e3)

     Mini = "{:1.0f}".format(M[0])

    # === IN LOOP PLOTS ===    

    # ::: HRD ::: 
     ax1.plot(Teff/1e3, logL, lw = 2, linestyle = '-', color = 'black', zorder=4)
     ims1 = ax1.scatter(Teff/1e3, logL,c= vrot, marker='o', edgecolors='none', s=300, cmap=plt.cm.gnuplot2, vmin = 0, vmax = 420, zorder = 3)
     
    # plot initial masses next to some tracks
     if i < 4 :  
      ax1.text(Teff[0]/1e3 + 2.5, logL[0] - 0.18, Mini, fontsize = 46)
     if i > 21 :   
      # left
      ax1.text(Teff[0]/1e3 + 3.8, logL[0] - 0.18, Mini, fontsize = 46)
      # right
      #:::ax1.text(Teff[0]/1e3 - 3.9, logL[0] - 0.03, Mini, fontsize = 46)
      #-ax1.text(Teff[0]/1e3 - 3., logL[0], Mini, fontsize = 46)

     # ::: KIEL :::  
     ax111.plot(Teff/1e3, logg, lw = 2, linestyle = '-', color = 'black', zorder=4)
     #ax111.text(Teff[0]/1e3 + 1., logg[0] + 0.05, Mini, fontsize = 16)
     ims111 = ax111.scatter(Teff/1e3,logg,c= vrot, marker='o', edgecolors='none', s=300, cmap=plt.cm.gnuplot2, vmin = 0, vmax = 420, zorder=3)
     #ax111.plot(Teff[da3]/1e3, logg[da3], linestyle = '', marker = 'X', markersize = 32, color = 'yellow', zorder=5)

     # ::: HUNTER :::  
     ax3.plot(vrot, lognit, lw=2, linestyle = '-', color =  'black', zorder = 30 - i)
     ims3 = ax3.scatter(vrot, lognit,c= logg, marker='o', edgecolors='none', s=300, cmap=plt.cm.gist_rainbow, vmin = 3.0, vmax = 4.4, zorder = 30 - i)
     #:::ax3.text(vrot[-1] - 25, lognit[-1], Mini, fontsize = 16)
     
     # show initial mass for 3 and 60 Msun tracks (and some others if feasible). 
     if i < 1 :  
        ax3.text(vrot[-1] - 130, lognit[-1] - 0.06 , Mini, fontsize = 46)
     #if i == 4 :  
     # ax3.text(vrot[-1] + 5, lognit[-1] + 0.1, Mini, fontsize = 46)
     #if i == 5 :  
     # ax3.text(vrot[-1] + 20, lognit[-1]+0.03, Mini, fontsize = 46)
     if i > 25 :  
      ax3.text(vrot[-1] - 50, lognit[-1] + 0.06 , Mini, fontsize = 46)
        #-ax3.text(vrot[-1] + 35, lognit[-1] - 0.1, Mini, fontsize = 46)
     
    # === OUT of loop isos ===

     # ::: show isos on HRD :::

    ax1.plot(Teffzams, logLzams, lw = 4, linestyle = '-', color = 'grey', alpha = 0.5,zorder = 11)
    ax1.plot(Teffiso2, logLiso2, lw = 4, linestyle = '-', color = 'grey', alpha = 0.5,zorder = 11)
    ax1.plot(Teffiso4, logLiso4, lw = 4, linestyle = '-', color = 'grey', alpha = 0.5,zorder = 11)
    ax1.plot(Teffiso6, logLiso6, lw = 4, linestyle = '-', color = 'grey', alpha = 0.5,zorder = 11)
    ax1.plot(Teffiso8, logLiso8, lw = 4, linestyle = '-', color = 'grey', alpha = 0.5,zorder = 11)
    ax1.plot(Teffiso10, logLiso10, lw = 4, linestyle = '-', color = 'grey', alpha = 0.5,zorder = 11)
    ax1.plot(Teffiso50, logLiso50, lw = 4, linestyle = '-', color = 'grey', alpha = 0.5,zorder = 11)

    # ::: initial mass on Kiel :::

    if Zini == 0.014 : 
        #:ax111.text(49.5,4., '60 M$_\odot$', color  = 'black', fontsize = 46)
        ax111.text(12.,4.41, '3 M$_\odot$', color  = 'black', fontsize = 46)
        ax111.text(49.5,4.3, '60 M$_\odot$', color  = 'black', fontsize = 46)
       #- ax111.text(14.,4.41, '3 M$_\odot$', color  = 'black', fontsize = 46)

    if Zini != 0.014 : 
       # ax111.text(51.5,4.33, '60 M$_\odot$', color  = 'black', fontsize = 46)
       # ax111.text(15.,4.5, '3 M$_\odot$', color  = 'black', fontsize = 46)
        ax111.text(55.5,4.4, '60 M$_\odot$', color  = 'black', fontsize = 46)
        ax111.text(17.,4.6, '3 M$_\odot$', color  = 'black', fontsize = 46)
   
   # ::: iso lines and text on Kiel :::
    if scheme == 'NOMAG': 
        ax111.plot(Teffisox1, loggisox1, lw = 4, linestyle = '-', color = 'grey', alpha = 0.5, zorder=11)
        ax111.plot(Teffisox15, loggisox15, lw = 4, linestyle = '-', color = 'grey', alpha = 0.5,zorder=11)
        ax111.plot(Teffisox2, loggisox2, lw = 4, linestyle = '-', color = 'grey', alpha = 0.5,zorder=11)
        ax111.plot(Teffisox3, loggisox3, lw = 4, linestyle = '-', color = 'grey', alpha = 0.5,zorder=11)
        ax111.plot(Teffisox5, loggisox5, lw = 4, linestyle = '-', color = 'grey',  alpha = 0.5,zorder=11)
        ax111.plot(Teffisox10, loggisox10, lw = 4, linestyle = '-', color = 'grey',  alpha = 0.5,zorder=11)

        ax111.text(Teffisox1[-1] - 10., loggisox1[-1] + 0.17, r'$\frac{\Omega_{\mathrm{core}}}{\Omega_{\mathrm{surf}}} = 1$', color  = 'grey', fontsize = 46)
        ax111.text(Teffisox15[-1]+6., loggisox15[-1] , '1.2', color  = 'grey', fontsize = 46)
        ax111.text(Teffisox2[-1]+3., loggisox2[-1] , '2', color  = 'grey', fontsize = 46)
        ax111.text(Teffisox3[-1]+3., loggisox3[-1] + 0.01, '3', color  = 'grey', fontsize = 46)
        ax111.text(Teffisox5[-1]+3., loggisox5[-1] , '5', color  = 'grey', fontsize = 46)
        ax111.text(Teffisox10[-1]+4., loggisox10[-1] + 0.01, '10', color  = 'grey', fontsize = 46)

        #: Mix2 ax111.text(Teffisox15[-1]+6., loggisox15[-1] , '1.2', color  = 'grey', fontsize = 46)
        #: Mix2 ax111.text(Teffisox2[-1]+5., loggisox2[-1] , '2', color  = 'grey', fontsize = 46)
        #---ax111.text(Teffisox15[0]-1.5, loggisox15[0] , '1.2', color  = 'grey', fontsize = 46)
        #:::   ax111.text(Teffisox2[0]-2., loggisox2[0] , '2', color  = 'grey', fontsize = 46)
        #---ax111.text(Teffisox2[0]+5., loggisox2[0] -0.05, '2', color  = 'grey', fontsize = 46)
    
    if scheme == 'INT': 
        ax111.plot(Teffisox1, loggisox1, lw = 4, linestyle = '-', color = 'grey', alpha = 0.5, zorder=11)
        #ax111.text(25, 3.3, r'$\frac{\Omega_{\mathrm{core}}}{\Omega_{\mathrm{surf}}} = 1$', color  = 'grey', fontsize = 46)
        ax111.text(55., 3.5, r'$\frac{\Omega_{\mathrm{core}}}{\Omega_{\mathrm{surf}}} = 1$', color  = 'grey', fontsize = 46)

    if scheme == 'SURF': 
        ax111.plot(Teffisox1, loggisox1, lw = 4, linestyle = '-', color = 'grey', alpha = 0.5, zorder=11)
        ax111.plot(Teffisox15, loggisox15, lw = 4, linestyle = '-', color = 'grey', alpha = 0.5,zorder=11)
        ax111.plot(Teffisox2, loggisox2, lw = 4, linestyle = '-', color = 'grey', alpha = 0.5,zorder=11)
        ax111.plot(Teffisox3, loggisox3, lw = 4, linestyle = '-', color = 'grey', alpha = 0.5,zorder=11)
        ax111.plot(Teffisox5, loggisox5, lw = 4, linestyle = '-', color = 'grey', alpha = 0.5,zorder=11)
        #ax111.plot(Teffisox10, loggisox10, lw = 4, linestyle = '-', color = 'grey', alpha = 0.5,zorder=11)

# right 
    #    ax111.text(Teffisox1[-1] - 10, loggisox1[-1] + 0.18, r'$\frac{\Omega_{\mathrm{core}}}{\Omega_{\mathrm{surf}}} = 1$', color  = 'grey', fontsize = 46)
    #    ax111.text(Teffisox15[0]-2., loggisox15[0] , '1.2', color  = 'grey', fontsize = 46)
    #    ax111.text(Teffisox2[0]-3., loggisox2[0] - 0.01 , '2', color  = 'grey', fontsize = 46)
    #    ax111.text(Teffisox3[0]-2., loggisox3[0] + 0.01, '3', color  = 'grey', fontsize = 46)
    #    ax111.text(Teffisox5[-1]+3., loggisox5[-1] , '5', color  = 'grey', fontsize = 46)

# left 
        ax111.text(Teffisox1[-1] - 10, loggisox1[-1] + 0.18, r'$\frac{\Omega_{\mathrm{core}}}{\Omega_{\mathrm{surf}}} = 1$', color  = 'grey', fontsize = 46)
        ax111.text(Teffisox15[-1]+7., loggisox15[-1] , '1.2', color  = 'grey', fontsize = 46)
        ax111.text(Teffisox2[-1]+4., loggisox2[-1] , '2', color  = 'grey', fontsize = 46)
        ax111.text(Teffisox3[-1]+3.6, loggisox3[-1] + 0.05, '3', color  = 'grey', fontsize = 46)
        ax111.text(Teffisox5[-1]+3., loggisox5[-1]-0.05, '5', color  = 'grey', fontsize = 46)
        #ax111.text(Teffisox10[-1]+4., loggisox10[-1] , '10', color  = 'grey', fontsize = 46)

    # +++++++++++++++++++++++++++++++++++++++++++++++
    # FINISH PLOTS:
    # BOX and more TEXT 
    # +++++++++++++++++++++++++++++++++++++++++++++++
    si = 46

    #strg = '\n'.join(('$B_{eq, ini} = $ %s kG', '$Z_{ini} = $ %s', r'$\frac{\Omega_{ini}}{\Omega_{crit, ini}}= 0.5$','SURF scheme', 'Mix1 scheme')) % (Bpinin, Zini)
    strg = '\n'.join(('$B_{\mathrm{eq, ini}} = $ %s kG', '$Z_{\mathrm{ini}} = $ %s', r'$\frac{\Omega_{\mathrm{ini}}}{\Omega_{\mathrm{crit, ini}}}= 0.5$', scheme+' scheme', mix+' scheme')) % (Bpinin, Zini)
    ax1.text(57., 2., strg, bbox=dict(facecolor='white', alpha=0.5, boxstyle='round'), fontsize = si, color = 'black')
    #ax1.text(69., 2., strg, bbox=dict(facecolor='white', alpha=0.5, boxstyle='round'), fontsize = si, color = 'black')

    # solar:
    if Zini == 0.014 : 
       ax111.text(54.,3., strg, bbox=dict(facecolor='white', alpha=0.5, boxstyle='round'), fontsize = si, color = 'black')
    #  NOMAG Mix2
     #--ax3.text(0., 7.8, strg, bbox=dict(facecolor='white', alpha=0.5, boxstyle='round'), fontsize = 38, color = 'black')
     # usual
       ax3.text(245, 9.05, strg, bbox=dict(facecolor='white', alpha=0.5, boxstyle='round'), fontsize = si, color = 'black')
    # LMC:
    if Zini == 0.0064 : 
   #:::  ax111.text(54., 3.27, strg, bbox=dict(facecolor='white', alpha=0.5, boxstyle='round'), fontsize = si, color = 'black')
   #:::  ax3.text(250, 7.95, strg, bbox=dict(facecolor='white', alpha=0.5, boxstyle='round'), fontsize = si, color = 'black')
       ax111.text(27., 3.21, strg, bbox=dict(facecolor='white', alpha=0.5, boxstyle='round'), fontsize = si, color = 'black')
   #  ax111.text(68., 3.21, strg, bbox=dict(facecolor='white', alpha=0.5, boxstyle='round'), fontsize = si, color = 'black')
   #  ax3.text(0., 7.15, strg, bbox=dict(facecolor='white', alpha=0.5, boxstyle='round'), fontsize = si, color = 'black')
       ax3.text(250., 9.0, strg, bbox=dict(facecolor='white', alpha=0.5, boxstyle='round'), fontsize = si, color = 'black')
    # SMC:
    if Zini == 0.0026 : 
       ax111.text(28., 3.2, strg, bbox=dict(facecolor='white', alpha=0.5, boxstyle='round'), fontsize = si, color = 'black')
       ax3.text(260., 8.65, strg, bbox=dict(facecolor='white', alpha=0.5, boxstyle='round'), fontsize = si, color = 'black')

    # ::: iso texts for HRD ::: 
    # solar
    if Zini == 0.014 : 
     ax1.text(53.5, 5.8, 'ZAMS', fontsize = si, color = 'grey')
     ax1.text(45., 5.95, '2 Myr', fontsize = si, color = 'grey')
     ax1.text(22.8, 5.74, '4 Myr', fontsize = si, color = 'grey')
     ax1.text(25., 5.4, '6 Myr', fontsize = si, color = 'grey')
     ax1.text(24., 5.15, '8 Myr', fontsize = si, color = 'grey')
     ax1.text(22.5, 4.9, '10 Myr', fontsize = si, color = 'grey')
     ax1.text(13.5, 3.25, '50 Myr', fontsize = si, color = 'grey')

   #:  ax1.text(54.5, 5.8, 'ZAMS', fontsize = si, color = 'grey')
   #  ax1.text(45., 5.98, '2 Myr', fontsize = si, color = 'grey')
   #  ax1.text(28., 6.04, '4 Myr', fontsize = si, color = 'grey', zorder = 44)
   #  ax1.text(38., 5.5, '6 Myr', fontsize = 26, color = 'grey')
   #  ax1.text(28., 5.4, '8 Myr', fontsize = 26, color = 'grey')
   #  ax1.text(20., 5.45, '10 Myr', fontsize = si, color = 'grey')
   #  ax1.text(20., 4., '50 Myr', fontsize = 26, color = 'grey')

    # iso texts for LMC
    if Zini == 0.0064 : 
     ax1.text(55.5, 5.83, 'ZAMS', fontsize = si, color = 'grey')
     ax1.text(50., 5.99, '2 Myr', fontsize = si, color = 'grey')
     ax1.text(27., 5.8, '4 Myr', fontsize = si, color = 'grey')
     ax1.text(28.5, 5.5, '6 Myr', fontsize = si, color = 'grey')
     ax1.text(26.5, 5.2, '8 Myr', fontsize = si, color = 'grey')
     ax1.text(23., 4.9, '10 Myr', fontsize = si, color = 'grey')
     ax1.text(15., 3.22, '50 Myr', fontsize = si, color = 'grey')

    # iso texts for SMC
    if Zini == 0.0026 : 
     ax1.text(39., 4., 'ZAMS', fontsize = si, color = 'grey')
     ax1.text(55.5, 6.04, '2 Myr', fontsize = si, color = 'grey')
     ax1.text(38., 6., '4 Myr', fontsize = si, color = 'grey')
     ax1.text(35., 5.55, '6 Myr', fontsize = si, color = 'grey')
     ax1.text(28.5, 5.3, '8 Myr', fontsize = si, color = 'grey')
     ax1.text(25.5, 4.9, '10 Myr', fontsize = si, color = 'grey')
     ax1.text(16., 3.3, '50 Myr', fontsize = si, color = 'grey')

    # ::: COLOR BARS :::  
    cbar = fig1.colorbar(ims1)
    cbar.ax.tick_params(labelsize=54) 
    #cbar.set_label('age [Myr]', size = 54)
    #cbar.set_label('log$\Omega / \Omega_{crit}$', size = 54)
    cbar.set_label('$v_{rot}$ [km s$^{-1}$]', size = 54)

    cbar = fig111.colorbar(ims111)
    cbar.ax.tick_params(labelsize=54) 
    cbar.set_label('$v_{rot}$ [km s$^{-1}$]', size = 54)

    cbar = fig3.colorbar(ims3)
    cbar.ax.tick_params(labelsize=54) 
    cbar.set_label('log $g$ [cm s$^{-2}$]', size = 54)

    fig1.tight_layout()
    fig111.tight_layout()
    fig3.tight_layout()

# save figures (make sure directories exist)
    fig1.savefig('Z'+Zin+'Om5/'+scheme+'/hrd'+mix+'/hrd'+Bpini+'.png')
    fig111.savefig('Z'+Zin+'Om5/'+scheme+'/kiel'+mix+'/kiel'+Bpini+'.png')
    fig3.savefig('Z'+Zin+'Om5/'+scheme+'/hunter'+mix+'/hunter'+Bpini+'.png')

# prodiction run is set up here: 
if __name__=="__main__":
    mix_list = ['Mix1']  # 'Mix1','Mix2'
    for mix in mix_list:
     Bp_list = 1.e-3 * np.array([0]) #1.e-3 * np.array([250, 500, 750, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500, 8000, 8500, 9000, 9500, 10000, 15000, 20000, 30000, 50000])
     for Bpinin in Bp_list:
        allplots(mix,Bpinin)