'''
This notebook is for creating isochrones of Zsolt's magnetic star models. 

Ylva Götberg, 3 December 2021
Sam Geen, June 2022
'''

# Set up some pyplot stuff
import customplot

# Import some packages
import numpy as np
import os
import copy
import glob
import matplotlib as mpl
import matplotlib.pyplot as plt
from astropy import constants as const
from astropy import units as u

# Allow for other directory with functions
import sys

# This one makes LaTeX in the plots
from matplotlib import rc
fsize = 20
rc('font',**{'family':'serif','serif':['Times'],'size'   : fsize})
rc('text', usetex=True)

def makefolders(folder):
    # Make folder structure if not already made, with debug option to just print it
    try:
        os.makedirs(folder)
    except:
        pass

def makeisochrones(infolder,outfolder):
    '''
    Make isochrones for one model
    folder = location of data (string, e.g. '/mnt/e/INT/Z14Mix1/')
    '''
    # Where is the grid? 
    print("Running for model folder",infolder)

    loc_grid = infolder+"/"
    main_grid = loc_grid.split('/')[-2]
    print(main_grid)

    # Get all the B-field strengths
    Bgrids = [name for name in os.listdir(loc_grid) if os.path.isfile(loc_grid+name+'/10.0/LOGS/out.data')]
    Bstrengths = np.float_(Bgrids)
    nbr_B = len(Bgrids)
    print("Running for B-fields",Bgrids)

    ind_sort = np.argsort(Bstrengths)
    Bgrids = np.array(Bgrids)[ind_sort]
    Bstrengths = Bstrengths[ind_sort]

    # Loop over the B-field grids and check what masses there are and read the MESA models

    # What to record
    col = ['star_age','log_L','log_Teff','log_g']
    # Let's use L, Teff, age, logg
    star_age = [None]*nbr_B
    log_L = [None]*nbr_B
    log_Teff = [None]*nbr_B
    log_g = [None]*nbr_B

    # Storage space
    Msims = [None]*nbr_B

    for i in range(nbr_B):
        
        print('Inside', Bgrids[i])
        
        # Check what masses this B-field strength has
        Msims[i] = [name for name in os.listdir(loc_grid+Bgrids[i]) if os.path.isfile(loc_grid+Bgrids[i]+'/'+name+'/LOGS/out.data')]
        
        # Sort with mass
        ind_sort = np.argsort(np.float_(Msims[i]))
        Msims[i] = np.array(Msims[i])[ind_sort]
        nbr_M = len(Msims[i])
        
        # Loop over the masses and read the MESA files
        star_age[i] = [None]*nbr_M
        log_L[i] = [None]*nbr_M
        log_Teff[i] = [None]*nbr_M
        log_g[i] = [None]*nbr_M
        for j in range(nbr_M):
            
            # where is the history file for this model?
            filename_history = loc_grid+Bgrids[i]+'/'+Msims[i][j]+'/LOGS/out.data'
            
            # Ylva version
            '''
            # Read the file
            data = GetColumnMESAhist(filename_history,col)
            star_age[i][j] = data[col.index('star_age')]
            log_L[i][j] = data[col.index('log_L')]
            log_Teff[i][j] = data[col.index('log_Teff')]
            log_g[i][j] = data[col.index('log_g')]
            
            # Find ZAMS and remove the pre-ZAMS part
            log_Lnuc = data[col.index('log_Lnuc')]
            ind = locate_ZAMS(log_L[i][j],log_Lnuc)

            star_age[i][j] = star_age[i][j][ind:]-np.min(star_age[i][j][ind:])
            log_L[i][j] = log_L[i][j][ind:]
            log_Teff[i][j] = log_Teff[i][j][ind:]
            log_g[i][j] = log_g[i][j][ind:]
            '''

            # Zsolt version
            data = np.genfromtxt(filename_history,skip_header=6)
            #-----------------------------------------------
            coreh = data[:,59]
            xd = coreh.compress((coreh>coreh[0]*0.997).flat)
            x = len(xd)

            model = data[:,0]
            m = len(model)
            age = data[x:,1]
            M = data[x:,2]
            logM = np.log10(M)
            logTeff = data[x:,6]
            Teff = data[x:,7]
            logL = data[x:,11]
            R = data[x:,13]
            logg = data[x:,14]

            star_age[i][j] = age - np.min(age) # data[col.index('star_age')]
            log_L[i][j] = logL # data[col.index('log_L')]
            log_Teff[i][j] = logTeff # data[col.index('log_Teff')]
            log_g[i][j] = logg # data[col.index('log_g')]
            
            #print('At M=',Msims[i][j]+'....')

    t_iso = np.array([0.,2.,4.,6.,8.,10.,50.])*1e6  # yrs
    nbr_t = len(t_iso)

    log_L_iso = [None]*nbr_B
    log_Teff_iso = [None]*nbr_B
    log_g_iso = [None]*nbr_B
    # Just for verification
    star_age_iso = [None]*nbr_B

    for i in range(nbr_B):
        
        nbr_M = len(Msims[i])
        
        log_L_iso[i] = -99.*np.ones([nbr_t,nbr_M])
        log_Teff_iso[i] = -99.*np.ones([nbr_t,nbr_M])
        log_g_iso[i] = -99.*np.ones([nbr_t,nbr_M])
        star_age_iso[i] = -99.*np.ones([nbr_t,nbr_M])
        for t in range(nbr_t):
            for j in range(nbr_M):
                if (np.max(star_age[i][j]) > t_iso[t]):
                    ind = np.argmin(np.abs(star_age[i][j]-t_iso[t]))
                    log_L_iso[i][t,j] = log_L[i][j][ind]
                    log_Teff_iso[i][t,j] = log_Teff[i][j][ind]
                    log_g_iso[i][t,j] = log_g[i][j][ind]
                    star_age_iso[i][t,j] = star_age[i][j][ind]

                    
            

        print('At ', Bgrids[i]+'...')

    # Where do we put this? How about I make one isochrone file per B-field strength? And put it in their 
    # grid folder? 

    for i in range(nbr_B):
        
        # Call it something
        #outfolder_B = outfolder+Bgrids[i]
        filename_iso = outfolder+'/isochrones_B'+Bgrids[i]+'.txt'
        
        # Open it
        fid = open(filename_iso,'w')
        # write a description
        fid.write('# This file contains the isochrones for the '+main_grid+' with B-field strength '+Bgrids[i]+'G.\n')
        fid.write('# All isochrones are here, and the values of log L, log Teff and log g are given for each time (given in Myr).\n')
        fid.write('# If a value is -99, there is no data for the star at that time (it left the main sequence).\n')
        
        # prepare the header
        header = 'Minit'
        for t in range(nbr_t):
            tt = str(int(t_iso[t]/1e6))
            header = header+' \t log_Teff_'+tt+' \t log_L_'+tt+' \t log_g_'+tt
        fid.write(header+'\n')
        
        # Now let's add the data - each line is for a model
        for j in range(len(Msims[i])):
            data_line = copy.copy(Msims[i][j])
            for t in range(nbr_t):
                data_line = data_line+'\t'+str(log_Teff_iso[i][t,j])+'\t'+str(log_L_iso[i][t,j])+'\t'+str(log_g_iso[i][t,j])
            fid.write(data_line+'\n')
        fid.close()

    # Let's pick all B-field strength
    for ii in range(0,len(Bgrids)):
        #ii = min(15,len(Bgrids)-1)
        print('Picking grid:',Bgrids[ii],'G')

        # Let's start with checking so all the ages were ok for the isochrones
        ti_mat = (np.array([t_iso]).T*np.ones(nbr_M))
        tmp = (star_age_iso[ii]-ti_mat)

        ind_check = np.abs(tmp) > 1e5
        print(np.sum(ind_check))
        print(ti_mat[ind_check])

        fig, ax = plt.subplots(1,1,figsize=(6,6))

        # Let's plot some tracks thin in the background
        #j = 10
        #print(Msims[i][j])
        for j in range(len(Msims[i])):
            ax.plot(log_Teff[ii][j],log_L[ii][j],'-',color=[.5]*3,lw=1)

        # Loop over the isochrones
        cmap = mpl.cm.get_cmap('plasma')
        clrs = cmap(np.linspace(0.1,.9,nbr_t))
        for t in range(nbr_t):
            ind_pres = log_Teff_iso[ii][t,:] != -99.
            ax.plot(log_Teff_iso[ii][t,:][ind_pres],log_L_iso[ii][t,:][ind_pres],'.',ms=10,linestyle='-',
                    color=clrs[t],label=str(int(t_iso[t]/1e6))+' Myr')

        ax.invert_xaxis()

        ax.set_xlabel('$\\log_{10} T_{\\mathrm{eff}}/\\mathrm{K}$')
        ax.set_ylabel('$\\log_{10} L/L_{\\odot}$')

        ax.tick_params(direction="in", which='both')
        ax.tick_params('both', length=6, width=1,which='major')
        ax.tick_params('both', length=3, width=1,which='minor')
        ax.xaxis.set_ticks_position('both')
        ax.yaxis.set_ticks_position('both')
        ax.legend(loc=0,fontsize=0.7*fsize,frameon=False)
        fig.savefig(outfolder+"/iso_"+Bgrids[ii]+".pdf")

    '''
    fig, ax = plt.subplots(1,1,figsize=(6,6))

    # Pick a time
    #tt = 2
    #print(t_iso[tt]/1e6,'Myr')

    # Loop over the B-fields
    #for i in range(0,nbr_B,2):

    cmap = mpl.cm.get_cmap('plasma')
    clrs = cmap(np.linspace(0.1,.9,nbr_B))
    for t in range(0,nbr_t,2):
        for i in range(nbr_B):
            ind_pres = log_Teff_iso[ii][t,:] != -99.
            ax.plot(log_Teff_iso[i][t,:][ind_pres],log_L_iso[i][t,:][ind_pres],'.',ms=10,linestyle='-',
                color=clrs[i])



    ax.invert_xaxis()

    ax.set_xlabel('$\\log_{10} T_{\\mathrm{eff}}/\\mathrm{K}$')
    ax.set_ylabel('$\\log_{10} L/L_{\\odot}$')

    ax.tick_params(direction="in", which='both')
    ax.tick_params('both', length=6, width=1,which='major')
    ax.tick_params('both', length=3, width=1,which='minor')
    ax.xaxis.set_ticks_position('both')
    ax.yaxis.set_ticks_position('both')
    ax.legend(loc=0,fontsize=0.7*fsize,frameon=False)
    fig.savefig("testisochrone.pdf")

    '''

def runforsuite(rootfolder):
    # Run for a named suite of data by diving into the data folders
    models = glob.glob("../../"+rootfolder+"/*")
    # LEVEL 1: Which model to use (Metallicity + Mixing)
    for model in models:
        if not "zip" in model:
            print("Running for model", model)
            outfolder = "../RDM/ISOCHRONES/"+model[6:] # double check where you write these
            print("Outputting to",outfolder)
            makefolders(outfolder)
            makeisochrones(model,outfolder)

def runall():
    runforsuite("INT")
    runforsuite("NOMAG")
    runforsuite("SURF")

if __name__=="__main__":
    runall()